local L = LibStub("AceLocale-3.0"):GetLocale("Talented")
local Talented = Talented

Talented.defaults = {
	profile = {

-- Always Edit the template, never lock
		-- always_edit = nil,

-- Manually confirm each talent before learning
		confirmlearn = true,

-- Do not confirm when applying a complete template
		confirmlearn_skip_on_apply = true,

-- Call LearnTalent() even if Talented has determined that it would not have any effect
		-- always_call_learn_talents = nil,

-- Report when Talented_Data is unavailable
		report_unavailable = true,

-- We have already reported that Talented_Data is unavailable
		-- reported_once = nil,

-- Limit Talented to the level cap
		level_cap = true,

-- Show the required level for any template
		show_level_req = true,

-- Offset between Talent icons
		offset = 64,

-- Global frame scale
		scale = 1,

-- the template that was selected last
	--	last_template = nil,

-- Force Talented_Data to be loaded even if incorrect version
		-- force_load = nil,

-- Frame Point relative to UIParent
		offsetx = 0,
		offsety = 0,
		point = "CENTER",

-- Do we hook the inspect UI to replace it by Talented ?
		-- hook_inspect_ui = nil,

-- Use a dialog to output URL or show them directly in Chat ?
		-- show_url_in_chat = nil,
	},
	global = {
	-- the list of saved templates, see below for the format
			templates = {},
	},
}

function Talented:SetOption(info, value)
	local name = info[#info]
	self.db.profile[name] = value
	local arg = info.arg
	if arg then self[arg](self) end
end

function Talented:GetOption(info)
	local name = info[#info]
	return self.db.profile[name]
end

function Talented:MustNotConfirmLearn()
	return not Talented.db.profile.confirmlearn
end

Talented.options = {
	desc = L["Talented - Talent Editor"],
	type = "group",
	childGroups = "tab",
	handler = Talented,
	get = "GetOption",
	set = "SetOption",
	args = {
		options = {
			name = L["Options"],
			desc = L["General Options for Talented."],
			type = "group",
			args = {
				always_edit = {
					name = L["Always edit"],
					desc = L["Always allow templates and the current build to be modified, instead of having to Unlock them first."],
					type = "toggle",
					set = "SetOption",
					arg = "UpdateView",
					order = 100,
				},
				confirmlearn = {
					name = L["Confirm Learning"],
					desc = L["Ask for user confirmation before learning any talent."],
					type = "toggle",
					order = 200,
				},
				confirmlearn_skip_on_apply = {
					name = L["Don't Confirm when applying"],
					desc = L["Don't ask for user confirmation when applying full template."],
					type = "toggle",
					disabled = "MustNotConfirmLearn",
					order = 210,
				},
				always_call_learn_talents = {
					name = L["Always try to learn talent"],
					desc = L["Always call the underlying API when a user input is made, even when no talent should be learned from it."],
					type = "toggle",
					disabled = "MustNotConfirmLearn",
					order = 220,
				},
				level_cap = {
					name = L["Talent cap"],
					desc = L["Restrict templates to a maximum of %d points."]:format(Talented.max_talent_points),
					type = "toggle",
					set = "SetOption",
					arg = "UpdateView",
					order = 300,
				},
				show_level_req = {
					name = L["Level restriction"],
					desc = L["Show the required level for the template, instead of the number of points."],
					type = "toggle",
					set = "SetOption",
					arg = "UpdateView",
					order = 400,
				},
				hook_inspect_ui = {
					name = L["Hook Inspect UI"],
					desc = L["Hook the Talent Inspection UI."],
					type = "toggle",
					arg = "CheckHookInspectUI",
					order = 700,
				},
				show_url_in_chat = {
					name = L["Output URL in Chat"],
					desc = L["Directly outputs the URL in Chat instead of using a Dialog."],
					type = "toggle",
					order = 750,
				},
				offset = {
					name = L["Icon offset"],
					desc = L["Distance between icons."],
					type = "range",
					min = 42,
					max = 64,
					step = 1,
					order = 800,
					arg = "ReLayout",
				},
				scale = {
					name = L["Frame scale"],
					desc = L["Overall scale of the Talented frame."],
					type = "range",
					min = 0.5,
					max = 1.0,
					step = 0.01,
					order = 900,
					arg = "ReLayout",
				},
			}
		},
	},
}


