local GLYPH_POSITIONS = {
	{ "CENTER", -15, 140, 0, 82 },
	{ "CENTER", -14, -103, 0, -82 },
	{ "TOPLEFT", 28, -133, -73, 44 },
	{ "BOTTOMRIGHT", -56, 168, 73, -44 },
	{ "TOPRIGHT", -56, -133, 73, 44 },
	{ "BOTTOMLEFT", 26, 168, -73, -44 },
}

local SparkleDimensions = { 7, 10, 13 }
local SparkleDurations = { 1.25, 3, 5.4 }
local function Sparkle_Update(self)
	local sparkle = self:GetParent()
	local index = math.random(#SparkleDimensions)
	local size = SparkleDimensions[index]
	sparkle:SetWidth(size)
	sparkle:SetHeight(size)
	self.translation:SetDuration(SparkleDurations[index])
end

local function MakeSparkleAnimation(parent, x, y)
	local sparkle = parent:CreateTexture(nil, "ARTWORK")
	sparkle:SetTexture"Interface\\Spellbook\\UI-GlyphFrame"
	sparkle:SetPoint("CENTER", 17, 18)
	sparkle:SetTexCoord(0.3984375, 0.4453125, 0.40234375, 0.44921875)
	sparkle:SetBlendMode"ADD"

	local animGroup = sparkle:CreateAnimationGroup(nil, "TalentedGlyphAnimTemplate")
	animGroup:SetLooping"REPEAT"
	local translation = animGroup.translation
	translation:SetOffset(x, y)
	animGroup:SetScript("OnFinished", Sparkle_Update)
	Sparkle_Update(animGroup)
	animGroup:Play()
	sparkle.anim = animGroup
	return sparkle
end

local function GlyphFrame_Update(self)
	for id, position in ipairs(GLYPH_POSITIONS) do
		local glyph = self.glyphs[id]
		if not glyph then
			glyph = Talented:MakeGlyph(self, id)
			self.glyphs[id] = glyph
			glyph.sparkle = MakeSparkleAnimation(self, unpack(position, 4))
		end
		glyph:ClearAllPoints()
		glyph:SetPoint(unpack(position, 1, 3))
		glyph:Update()
	end
end

local function GlyphFrame_OnEvent(self, event, ...)
	if event == "GLYPH_ADDED" or event == "GLYPH_UPDATED" or event == "GLYPH_REMOVED" then
		local id = ...
		local glyph = self.glyphs[id]
		glyph:Update()
		local _, type = GetGlyphSocketInfo(id, self.group)
		if event == "GLYPH_REMOVED" then
			PlaySound(type == 2 and "Glyph_MinorDestroy" or "Glyph_MajorDestroy")
		else
			PlaySound(type == 2 and "Glyph_MinorCreate" or "Glyph_MajorCreate")
		end
		self.glow:Show()
	else
		self:Update()
	end
end

local function GlyphFrame_OnEnter(self)
	if SpellIsTargeting() then SetCursor"CAST_ERROR_CURSOR" end
end

local function GlyphFrame_OnLeave(self)
	SetCursor(nil)
end

function Talented:MakeGlyphFrame(parent)
	local frame = CreateFrame("Frame", "TalentedGlyphs", parent)

	frame:SetFrameStrata("DIALOG")
	frame:SetWidth(384)
	frame:SetHeight(512)
	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:SetToplevel(true)

	local t = frame:CreateTexture(nil, "ARTWORK")
	t:SetTexture"Interface\\Spellbook\\UI-GlyphFrame"
	t:SetWidth(352)
	t:SetHeight(441)
	t:SetPoint"TOPLEFT"
	t:SetTexCoord(0, 0.6875, 0, 0.861328125)

	local p = frame:CreateTexture()
	p:SetWidth(64)
	p:SetHeight(64)
	SetPortraitTexture(p, "player")
	p:SetPoint("TOPLEFT", 5, -4)
	frame.portrait = p

	local fs = frame:CreateFontString(nil, "ARTWORK")
	fs:SetFontObject(GameFontNormal)
	fs:SetText(GLYPHS)
	fs:SetPoint("TOP", 0, -17)
	frame.title = fs

	local glow = frame:CreateTexture(nil, "OVERLAY")
	glow:Hide()
	glow:SetWidth(352)
	glow:SetHeight(441)
	glow:SetPoint("TOPLEFT", -9, -38)
	glow:SetTexCoord(0, 0.6875, 0, 0.861328125)
	frame.glow = glow

	frame:RegisterEvent"GLYPH_ADDED"
	frame:RegisterEvent"GLYPH_REMOVED"
	frame:RegisterEvent"GLYPH_UPDATED"
	frame:RegisterEvent"GLYPHFRAME_OPEN"
	frame:RegisterEvent"PLAYER_LEVEL_UP"

	frame:SetScript("OnEvent", GlyphFrame_OnEvent)
	frame:SetScript("OnEnter", GlyphFrame_OnEnter)
	frame:SetScript("OnLeave", GlyphFrame_OnLeave)
	frame:SetScript("OnMouseDown", function(self) self:StartMoving() end)
	frame:SetScript("OnMouseUp", function(self) self:StopMovingOrSizing() Talented:SaveFramePosition() end)

	frame.Update = GlyphFrame_Update

	frame.glyphs = {}

	frame:SetPoint("CENTER", UIParent)
	frame:SetFrameLevel(TalentedFrame:GetFrameLevel() + 10)

	local close = CreateFrame("Button", nil, frame)
	frame.close = close

	local makeTexture = function (path, blend)
		local t = close:CreateTexture()
		t:SetAllPoints(close)
		t:SetTexture(path)
		if blend then
			t:SetBlendMode(blend)
		end
		return t
	end

	close:SetNormalTexture(makeTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up"))
	close:SetPushedTexture(makeTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down"))
	close:SetHighlightTexture(makeTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight", "ADD"))
	close:SetScript("OnClick", function (self) self:GetParent():Hide() end)

	close:SetWidth(32)
	close:SetHeight(32)
	close:ClearAllPoints()
	close:SetPoint("TOPRIGHT", -28, -9)
	return frame
end

function Talented:OpenGlyphFrame(group)
	local f = self:MakeGlyphFrame(self.base)
	f.group = group
	f:Update()
	f:Show()
	self.glyphs = f
	self.OpenGlyphFrame = function (self, group)
		local f = self.glyphs
		f.group = group
		f:Update()
		f:Show()
	end
end
